// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Shell.h"
#include "colors.h"

#include "ShellDoc.h"
#include "ShellView.h"
#include "BoardView.h"

#include "MainFrm.h"
#include "MsgQ.h"

#include "comb.h"
#include "threads.h"
#include "GCd.h"
#include "Splash.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
ON_UPDATE_COMMAND_UI(IDS_ACTIVE_THREADS, OnUpdateActiveThreads)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_COMMAND(ID_PEN_WIDTH_NARROW, OnPenWidthNarrow)
	ON_COMMAND(ID_PEN_WIDTH_MEDIUM, OnPenWidthMedium)
	ON_COMMAND(ID_PEN_WIDTH_THICK, OnPenWidthThick)
	ON_COMMAND(ID_PEN_COLOR, OnPenColor)
	ON_WM_CREATE()
	ON_COMMAND(ID_BUTTON_RECT, OnButtonRect)
	ON_COMMAND(ID_BUTTON_LINE, OnButtonLine)
	ON_COMMAND(ID_BUTTON_FREE, OnButtonFree)
	ON_COMMAND(ID_BUTTON_MOVE, OnButtonMove)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_RECT, OnUpdateButtonRect)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_LINE, OnUpdateButtonLine)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_FREE, OnUpdateButtonFree)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_MOVE, OnUpdateButtonMove)
	ON_WM_CLOSE()
	ON_COMMAND(ID_BUTTON_BLACK, OnButtonBlack)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_BLACK, OnUpdateButtonBlack)
	ON_COMMAND(ID_BUTTON_BLUE, OnButtonBlue)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_BLUE, OnUpdateButtonBlue)
	ON_COMMAND(ID_BUTTON_GREEN, OnButtonGreen)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_GREEN, OnUpdateButtonGreen)
	ON_COMMAND(ID_BUTTON_RED, OnButtonRed)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_RED, OnUpdateButtonRed)
	ON_COMMAND(ID_BUTTON_WHITE, OnButtonWhite)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_WHITE, OnUpdateButtonWhite)
	ON_COMMAND(ID_BUTTON_YELLOW, OnButtonYellow)
	ON_UPDATE_COMMAND_UI(ID_BUTTON_YELLOW, OnUpdateButtonYellow)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USER_STATUSCHANGED, OnStatusChanged)
END_MESSAGE_MAP()

HWND StatusBarHWND;

#define NARROW_PEN 1
#define MEDIUM_PEN 3
#define THICK_PEN  7

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction
   
CMainFrame::CMainFrame()
{
   m_UserPenW = MEDIUM_PEN;
   m_pUserPen = new CPen(PS_SOLID, m_UserPenW, COLOR_BLACK);  // the user free hand pen
   m_DrawShape = DRAW_FREE;     // free hand 
   m_BarColor = COLOR_BLACK;
}

 CMainFrame::~CMainFrame()
{
	delete m_pUserPen;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	return CMDIFrameWnd::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
extern "C" UINT SCM_FirstThread(LPVOID);

extern CShellView *MeanWhile_TheActiveView;


HWND hMainFrame=0; // This is here so other threads will be able to send the mainframe a message when status changed

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create toolbar
	m_ToolBar.Create(this);   
	m_ToolBar.LoadToolBar(IDR_TOOLBAR_DRAW);
	m_ToolBar.EnableToolTips();

	// Create statusbar
	theStatusBar.Create(this);
	UINT indicators[2]= { ID_SEPARATOR/*(Menus description)*/, IDS_ACTIVE_THREADS };
	theStatusBar.SetIndicators( &indicators[0], 2 );
	theStatusBar.SetPaneStyle(0,SBPS_STRETCH);
	theStatusBar.SetPaneStyle(1,SBPS_NOBORDERS);

	StatusBarHWND = theStatusBar.m_hWnd;
	hMainFrame = m_hWnd;
//	theStatusBar.SetPaneInfo(1, IDS_ACTIVE_THREADS, SBPS_NOBORDERS, 10);

	
	// CG: The following line was added by the Splash Screen component.
	CSplashWnd::ShowSplashScreen(this);
	return 0;
}

extern CShellApp theApp;
void CMainFrame::OnUpdateActiveThreads(CCmdUI *pUI)
{
	pUI->Enable();
}

CStatusBar theStatusBar;

// This does not much help
void UpdateStatusBar()
{ 
	::PostMessage(hMainFrame, WM_USER_STATUSCHANGED, 0,0);
}

LRESULT CMainFrame::OnStatusChanged(WPARAM wParam, LPARAM lPrarm)
{
	CString s;
	switch (The_GC_deamon_status)
	{
	case GC_STATUS_IDLE:
		s.Format(IDS_ACT_THRDS_FMT, Threads);
		break;
	case GC_STATUS_MARK:
		s.LoadString(IDS_STATUS_GCMARK);
		break;
	case GC_STATUS_SWEEP:
		s.LoadString(IDS_STATUS_GCSWEEP);
		break;
	}
	theStatusBar.SetPaneText(1, s);
	theStatusBar.UpdateWindow();
	return 0;
}

// Toolbar buttons

void CMainFrame::OnButtonRect()  { m_DrawShape = DRAW_RECT;  }
void CMainFrame::OnButtonFree()  { m_DrawShape = DRAW_FREE;  }
void CMainFrame::OnButtonLine()  { m_DrawShape = DRAW_LINE;  }
void CMainFrame::OnButtonMove()  { m_DrawShape = MOVE_TURTLE;}



void CMainFrame::OnUpdateButtonRect(CCmdUI* pCmdUI) { pCmdUI->SetCheck(m_DrawShape==DRAW_RECT  ); }
void CMainFrame::OnUpdateButtonFree(CCmdUI* pCmdUI) { pCmdUI->SetCheck(m_DrawShape==DRAW_FREE  ); }
void CMainFrame::OnUpdateButtonLine(CCmdUI* pCmdUI) { pCmdUI->SetCheck(m_DrawShape==DRAW_LINE  ); }
void CMainFrame::OnUpdateButtonMove(CCmdUI* pCmdUI) { pCmdUI->SetCheck(m_DrawShape==MOVE_TURTLE); }





void CMainFrame::OnPenColor() 
{  LOGPEN Log;
   
   m_pUserPen->GetLogPen(&Log);
   CColorDialog dColor(Log.lopnColor);
   if (dColor.DoModal() == IDOK)
   { if (Log.lopnColor != dColor.GetColor())  // a change
     { delete m_pUserPen;
       m_BarColor = Log.lopnColor = dColor.GetColor();
       m_pUserPen = new CPen(Log.lopnStyle, Log.lopnWidth.x, Log.lopnColor);
     }
   }
	
}



void CMainFrame::OnPenWidthNarrow()
{ LOGPEN Log;
  m_pUserPen->GetLogPen(&Log);
  delete m_pUserPen;
  m_pUserPen = new CPen(Log.lopnStyle, NARROW_PEN, Log.lopnColor);
}

void CMainFrame::OnPenWidthMedium()
{ LOGPEN Log;
  m_pUserPen->GetLogPen(&Log);
  delete m_pUserPen;
  m_pUserPen = new CPen(Log.lopnStyle, MEDIUM_PEN, Log.lopnColor);
}

void CMainFrame::OnPenWidthThick()
{ LOGPEN Log;
  m_pUserPen->GetLogPen(&Log);
  delete m_pUserPen;
  m_pUserPen = new CPen(Log.lopnStyle, THICK_PEN, Log.lopnColor);
}


void CMainFrame::OnClose() 
{	extern CShellApp theApp;
	CDocTemplate *pDT = (CDocTemplate *)(theApp.m_pInterpreterDocTemplate);
	POSITION Pos;
	Pos = pDT->GetFirstDocPosition();
	if (Pos && (pDT->GetNextDoc(Pos) != NULL))
	{ 
		AfxMessageBox(IDS_THREADS_STILL_ACTIVE, MB_OK | MB_ICONSTOP);
		return;
	}
	
	CMDIFrameWnd::OnClose();
}


void CMainFrame::SetUserPenColor(COLORREF Color)
{  LOGPEN Log;
   m_pUserPen->GetLogPen(&Log);
   delete m_pUserPen;
   Log.lopnColor = Color;
   m_pUserPen = new CPen;
   m_pUserPen->CreatePenIndirect(&Log);
   m_BarColor = Color;
}

void CMainFrame::OnButtonBlack() 
{  SetUserPenColor(COLOR_BLACK);
}

void CMainFrame::OnUpdateButtonBlack(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_BarColor == COLOR_BLACK);
}

void CMainFrame::OnButtonBlue() 
{ SetUserPenColor(COLOR_BLUE);
}

void CMainFrame::OnUpdateButtonBlue(CCmdUI* pCmdUI) 
{ pCmdUI->SetCheck(m_BarColor == COLOR_BLUE);
}

void CMainFrame::OnButtonGreen() 
{ SetUserPenColor(COLOR_GREEN);
}

void CMainFrame::OnUpdateButtonGreen(CCmdUI* pCmdUI) 
{ pCmdUI->SetCheck(m_BarColor == COLOR_GREEN);
}

void CMainFrame::OnButtonRed() 
{  SetUserPenColor(COLOR_RED);
} 	


void CMainFrame::OnUpdateButtonRed(CCmdUI* pCmdUI) 
{ pCmdUI->SetCheck(m_BarColor == COLOR_RED);
}

void CMainFrame::OnButtonWhite() 
{  SetUserPenColor(COLOR_WHITE);
}

void CMainFrame::OnUpdateButtonWhite(CCmdUI* pCmdUI) 
{ pCmdUI->SetCheck(m_BarColor == COLOR_WHITE);
}

void CMainFrame::OnButtonYellow() 
{  SetUserPenColor(COLOR_YELLOW);
}

void CMainFrame::OnUpdateButtonYellow(CCmdUI* pCmdUI) 
{ pCmdUI->SetCheck(m_BarColor == COLOR_YELLOW);
}
